/*
 *  Author: Pawel Parys <parys@mimuw.edu.pl> (2003-2004)
 *
 *  For licence see doc/LICENCE.
 */

#include <cstdio>
#include "config.h"
#include "string.h"
#include <string>
#include <vector>
#include "defines.h"
#include "tools.h"
#include "xxx.h"

/* Format of the configuration file is:
 *  - A declaration is: NAME=VALUE
 *  - VALUE is an integer.
 *  - NAME is one of allowed names of variables.
 *  - Case doesn't matter.
 *  - Single declaration have to be in single line.
 *  - A line can contain one or zero declarations.
 *  - All spaces are ignored (even these in the middle of a name or number).
 *  - All characters from '#' to end of line are skipped. */

/* File config_def.h defines variables which can be set in configuration file.
 * Lines of this file are calls of CONFIG macro with four parameters: 
 * 1) variable name in program, 
 * 2) variable name in configuration file,
 * 3) default value (two meanings - value in program when not set in
 *    configuration file and value written in default configuration file),
 * 4) comment - meaning of this variable (written in default config). 
 * Basing on config_def.h script generatecfg.pl automaticaly generates
 * default configuration file. Because of that all lines should have the
 * following format, spaces are allowed only after commas. */

using namespace std;

#define CONFIG_FILE "ttsolve.cfg" // name of the config file

vector<string> variableNames;	// this vector will contain names of variables
				// allowed in configuration file 
vector<int*> variables;	// these are pointers to corresponding variables in 
			// program

int strpos(const string &a, const string &b)
/* Searches for 'b' in 'a', return position or -1 when not found. */
{
	char *x = strstr(a.c_str(), b.c_str());
	return x==NULL ? -1 : x-a.c_str();
}

void skipSpaces(string &str)
/* Removes all spaces from 'str'. */
{
	int b = 0;
	REPV(a, str)
		if (!isspace(str[a]))
			str[b++]=str[a];
	str.resize(b);
}

void loadConfig()
{
	FILE *f = fopen(CONFIG_FILE, "r");
	if (f==NULL)
	{
		show(10,("No configuration file (%s)\n", CONFIG_FILE));
		return;
	}
	show(10,("Reading configuration file: %s\n", CONFIG_FILE));
	#define CONFIG(v, n, d, skip) variableNames.push_back(string(#n)); \
		extern int v; v = d; variables.push_back(&v);
	#include "config_def.h"
	int err = 0, line = 0, eline = 0;
	for (;;)
	{
		int ok;
		string left;
		line++;
		char buf[4096];
		char *res = fgets(buf, sizeof(buf), f);
		if (res==NULL || !buf[0]) break;
		string str = buf;
		int a = strpos(str, "#");
		if (a>=0) 
			str = str.substr(0, a);
		skipSpaces(str);
		if (str.empty()) continue;
		a = strpos(str, "=");
		if (a<0) goto error;
		left = str.substr(0, a);
		str = str.substr(a+1);
		for (a=0; ; a++)
		{
			if (a==size(variableNames)) goto error;
			if (strcasecmp(left.c_str(), variableNames[a].c_str())==0) break;
		}
//		if (a<ilestr)
//		{
//			if (str.Length()<2||str[1]!='"'||str[str.Length()]!='"') goto error;
//			*stringi[a]=str.SubString(2,str.Length()-2);
		ok = sscanf(str.c_str(), "%d", variables[a]);
		if (ok!=1) goto error;
		continue;
	    error:
		if (!err) { eline=line; err=1; }
	}
	fclose(f);
	if (err) 
		show(10,("Error while reading configuration file in line %d\n", eline));
}

