/*
 *  Author: Pawel Parys <parys@mimuw.edu.pl> (2003-2004)
 *
 *  For licence see doc/LICENCE.
 */

#ifndef __DEFINES_H
#define __DEFINES_H

#include <cstdio>

/* Basic definition used in program. */

// Enable (1) / disable (0) some features.
#define ENABLE_WEEKS 1	// Possible differences in even / odd weeks.
#define ENABLE_CONT 1	// 'Countinously' option available for resources.
#define ENABLE_MULTI 1	// Several concurrent processes.
#define ENABLE_DEBUG 1	// Some additional (debuging) checks (slows down).

#ifdef _WIN32
#undef ENABLE_MULTI
#define ENABLE_MULTI 0
#endif

/* For every of these options (OPT) we have tree macros.
 * OPT_ONLY(x) leaves x when OPT is on, else nothing.
 * OPT_ONLY(x,y) - x when OPT on, y when off.
 * OPT_ARG(x) - when OPT is on additional argument to function (comma and x).
 */
   
#if ENABLE_WEEKS
#define WEEKS_ONLY(x) x
#define WEEKS_ONLY2(x,y) x
#define WEEKS_ARG(x) , x
#else
#define WEEKS_ONLY(x) 
#define WEEKS_ONLY2(x,y) y
#define WEEKS_ARG(x)
#endif

#if ENABLE_CONT
#define CONT_ONLY(x) x
#define CONT_ONLY2(x,y) x
#define CONT_ARG(x) , x
#else
#define CONT_ONLY(x) 
#define CONT_ONLY2(x,y) y
#define CONT_ARG(x)
#endif

#if ENABLE_MULTI
#define MULTI_ONLY(x) x
#define MULTI_ONLY2(x,y) x
#define MULTI_ARG(x) , x
#else
#define MULTI_ONLY(x) 
#define MULTI_ONLY2(x,y) y
#define MULTI_ARG(x)
#endif

#if ENABLE_DEBUG
#define DEBUG_ONLY(x) x
#define DEBUG_ONLY2(x,y) x
#define DEBUG_ARG(x) , x
#else
#define DEBUG_ONLY(x) 
#define DEBUG_ONLY2(x,y) y
#define DEBUG_ARG(x)
#define NDEBUG 1
#endif

#define UNASSIGNED_SCORE 1000000	// score for unassigned activity
#if ENABLE_CONT
#define CONTINUOUS_SCORE  10000	// score for noncontinous timetable 
				// (for any "hole" one such unit)
#endif
#define DEFAULT_SOFT_SCORE 16	// score for soft preferences with no score
				// defined

extern bool showScore; 	// Set when -s option is given. Declared in main.cc
extern int verboseMode; // How much messages display. More means more. All
			// messages with lower or equal importance level are
			// displayes. 10 is normal, 20 is almost everything.
			// Declared in main.cc.

#define show(lev, arg) do { if (lev<=verboseMode) xprintf arg; } while (0)
// This macro should be used for displaing messages. First argument is
// importance level of the message. Second are, surrounded in parenthesis,
// arguments to printf.

/* And two simple but usefull functions. */

inline void updateMax(int &a, int b)
{
	if (b>a) a=b;
}

inline void updateMin(int &a, int b)
{
	if (b<a) a=b;
}

extern std::FILE *stderr2;

void xprintf(const char *str, ...);	// fprintf to stderr2. Declared in
					// inputparser.y. We need such
					// function in 'show'.

#endif
