/*
 *  Author: Pawel Parys <parys@mimuw.edu.pl> (2003-2004)
 *
 *  For licence see doc/LICENCE.
 */

/* This part of program is a greedy algorithm. It takes every single
 * activity and moves it to all possible start times. */

#include "xxx.h"
#include "myspace.h"
#include <climits>
#include <cassert>
#include <algorithm>

namespace Spaces
{

int firstTime = 0; 
// Only for debugging aims (when 1 some assert don't have to be meet).

vector<vector<char> > MySpace::FSResUsed;  //[res][time]
/* For each resource and for each timeslot we remember here if the resource
 * is used at this time (activity or break after activity). In two-weeks
 * version bit 0 describes odd weeks, bit 1 even week. In one-week version
 * only bit 0 is used. Value 1 meens used, value 0 unused. Additional value
 * IMPOSSIBLE (=4) meens that this resource can not be used at this time
 * (hard constraints). We exclude the group resources for activity which we
 * are moving currently. */



#define IMPOSSIBLE 4

#if ENABLE_CONT
DEBUG_ONLY2(,inline) void MySpace::FSInsertCont(int from, int to, int r WEEKS_ARG(int which))
{
	CREF(res, ps.resources[r]);
	myScore -= CalculateContinuousScore(res WEEKS_ARG(which), FScont_start, FScont_stop)
		*(res.continuousScore WEEKS_ONLY(/2));
	assert(firstTime || myScore>=0);
	updateMin(FScont_start WEEKS_ONLY([which]) [res.contres][from/ps.slotsPerDay], from);
	updateMax(FScont_stop WEEKS_ONLY([which]) [res.contres][from/ps.slotsPerDay], to);
	myScore += CalculateContinuousScore(res WEEKS_ARG(which), FScont_start, FScont_stop)
		*(res.continuousScore WEEKS_ONLY(/2));
}

DEBUG_ONLY2(,inline) void MySpace::FSRemoveCont(int from, int to, int r WEEKS_ARG(int which))
{
	CREF(res, ps.resources[r]);
	myScore -= CalculateContinuousScore(res WEEKS_ARG(which), FScont_start, FScont_stop)
		*(res.continuousScore WEEKS_ONLY(/2));
	assert(firstTime || myScore>=0);
	REF(start, FScont_start WEEKS_ONLY([which]) [res.contres][from/ps.slotsPerDay]);
	REF(stop, FScont_stop WEEKS_ONLY([which]) [res.contres][from/ps.slotsPerDay]);
	WEEKS_ONLY( int flag = which ? ODD_WEEKS : EVEN_WEEKS; )
	if (start==from)
	{
		start = to+1;
		for (; start<(from/ps.slotsPerDay+1)*ps.slotsPerDay; ++start)
			if (FSResUsed[r][start] WEEKS_ONLY(& flag))
				break;
	}
	if (stop==to)
	{
		stop = from-1;
		for (; stop>=from/ps.slotsPerDay*ps.slotsPerDay; --stop)
			if (FSResUsed[r][stop] WEEKS_ONLY(& flag))
				break;
	}
	myScore += CalculateContinuousScore(res WEEKS_ARG(which), FScont_start, FScont_stop)
		*(res.continuousScore WEEKS_ONLY(/2));
}
#endif

struct ResScore
{
	int *array;
	int *begins;
	int *operator[](int index) { return array+begins[index]; }
	~ResScore() { delete[] array; delete[] begins; }
};
/* It works as a two dimentional array:
 * [group number for activity][resource number in group]
 * The whole array is counted for specified activity act.
 * It counted assuming that act has length curLength and actually selected start time.
 * In each cell we remember a score which would come from this resource (its soft 
 * constraints) when it is selected by act.
 * For each hard costraint we add UNASSIGNED_SCORE. */

void MySpace::FSUpdateResScore(int a, int t0, int len, int dir, ResScore &resScore)
// Change resScore when allocating/disallocating timeslots t0 <-> t0+len-1 for
// activity a. When allocating dir=1, when disallocating dir=-1.
// It should be called when FSResUsed coresponding to these timeslots is not 
// allocated (before inserting / after removing).
{
	if (!firstTime) show(20,("          resScore changes to:"));
	CREF(act, ps.activities[a]);
	REPV(gi, act.requiredGroups)
	{
		int g = act.requiredGroups[gi];
		REPV(ri, ps.groups[g].resources)
		{
			int r = ps.groups[g].resources[ri];
			const PS::TimePreferences tp = ps.resources[r].timePreferences;
			FOR(t, t0, t0+len-1)
			{
				if (FSResUsed[r][t] WEEKS_ONLY( &(act.weekFlag|IMPOSSIBLE) ))
					resScore[gi][ri] += dir*UNASSIGNED_SCORE;
				resScore[gi][ri] += dir*tp.soft[t];
			}
			if (!firstTime)
			{
				show(20,(" %d", resScore[gi][ri]));
				assert(resScore[gi][ri]>=0);
			}
		}
	}
	if (!firstTime)
		show(20,("\n"));
}

void MySpace::FSUpdateMyScoreFromRes(int a, int dir, ResScore &resScore)
// Count scores from time constraints for group resources of activity a into myScore.
// dir = +1/-1 -> add/remove the score
{
	show(20,("      Updating myScore using resScore:"));
	CREF(act, ps.activities[a]);
	WEEKS_ONLY( int divide = act.weekFlag==ALL_WEEKS ? 1 : 2 );
	REPV(gi, act.requiredGroups)
	{
		int g = act.requiredGroups[gi];
		REPV(ri, ps.groups[g].resources)
		{
			if (ps.groups[g].resources[ri]==FSgrsel[act.grsel+gi])
				myScore += dir*resScore[gi][ri] WEEKS_ONLY(/divide);
			show(20,(" %d", resScore[gi][ri]));
		}
	}
	show(20,("\n"));
	assert(firstTime || myScore>=0);
}

void MySpace::FSRemoveAct(int a, int oldT, int len, bool breakReq, ResScore &resScore)
// Disallocate timeslots oldT <-> oldT+len-1 which were used for activity a.
// If breakReq then after them there is a break and will be disallocated too.
// It doesn't change start time and lenght of a.
// It changes FSResUsed, myScore and resScore. Changes of myScore doesn't take
//    into account score from soft constraints of group resources. These 
//    changes should be done later manually using resScore.
{
	assert(len>0||(len==0&&breakReq));
	show(20,("        Removing activity %d [%s], time: (%d-%d%s)\n", ACT(a), oldT, oldT+len-1, breakReq?"+break":""));
	show(20,("          myScore before: %d\n", myScore));
	CREF(act, ps.activities[a]);

	// changes to myScore from soft preferences (activity contains also 
	// resources preferences)
	FOR(t, oldT, oldT+len-1)
		myScore -= act.timePreferences.soft[t];
	
	// free resources
	FOREACH(r, act.requiredResources)
	{
		CREF(res, ps.resources[r]);
		int endT = oldT+len-1+(breakReq?res.breakLen:0);
		FOR(t, oldT, endT)
		{
			FSResUsed[r][t] &= WEEKS_ONLY2(~act.weekFlag, 0);
			show(30,("          FSResUsed[%d [%s]][%d]: %d\n", RES(r), t, FSResUsed[r][t]));
		}
		CONT_ONLY(
		if (res.contres>=0)
		{
			WEEKS_ONLY2(
			if (act.weekFlag & EVEN_WEEKS)
				FSRemoveCont(oldT, endT, r, 0);
			if (act.weekFlag & ODD_WEEKS)
				FSRemoveCont(oldT, endT, r, 1);
			,
			FSRemoveCont(oldT, endT, r);
			)
		}
		)
	}
	show(20,("          myScore after: %d\n", myScore));
	FSUpdateResScore(a, oldT, len, -1, resScore);
	assert(firstTime || myScore>=0);
}

void MySpace::FSInsertAct(int a, int newT, int len, bool breakReq, ResScore &resScore)
// Allocate timeslots newT <-> newT+len-1 for activity a.
// (Assuming these timeslots are free.)
// If breakReq break will be allocated after them.
// It doesn't change start time and lenght of a.
// It changes FSResUsed, myScore and resScore. Changes of myScore doesn't take
//    into account score from soft constraints of group resources. These 
//    changes should be done later manually using resScore.
{
	assert(len>0||(len==0&&breakReq));
	show(20,("        Inserting activity %d [%s], time: (%d-%d%s)\n", ACT(a), newT, newT+len-1, breakReq?"+break":""));
	FSUpdateResScore(a, newT, len, +1, resScore);
	show(20,("          myScore before: %d\n", myScore));
	CREF(act, ps.activities[a]);
		
	// changes to myScore from soft preferences (activity contains also 
	// resources preferences)
	FOR(t, newT, newT+len-1)
		myScore += act.timePreferences.soft[t];

	// allocate resources
	FOREACH(r, act.requiredResources)
	{
		CREF(res, ps.resources[r]);
		int endT = newT+len-1+(breakReq?res.breakLen:0);
		FOR(t, newT, endT)
		{
			FSResUsed[r][t] WEEKS_ONLY2( |= act.weekFlag, |= 1);
			show(30,("          FSResUsed[%d [%s]][%d]: %d\n", RES(r), t, FSResUsed[r][t]));
		}
		CONT_ONLY(
		if (res.contres>=0)
		{
			WEEKS_ONLY2(
			if (act.weekFlag & EVEN_WEEKS)
				FSInsertCont(newT, endT, r, 0);
			if (act.weekFlag & ODD_WEEKS)
				FSInsertCont(newT, endT, r, 1);
			,
			FSInsertCont(newT, endT, r);
			)
		}
		)
	}
	show(20,("          myScore after: %d\n", myScore));
}

int MySpace::FSInsertIfPossible(int a, int newT, int len, bool breakReq, int &ok, ResScore &resScore) 
// Try to allocale timeslots newT <-> newT+len-1 for activity a.
// If 'breakReq' then after that the break should be allocated.
// Possible -> allocate them, ok=1
// Impossible -> ok=0 and return next time at which we can try (first after detected collisions)
{
	assert(len>0||(len==0&&breakReq));
	ok = 0;
	CREF(act, ps.activities[a]);
	
	// check for activity hard constraints
	CREF(hp, act.timePreferences.hard);
	VAR(ti, lower_bound(ALL(hp), newT+len)); // find first number >= newT+len
	if (ti!=hp.begin() && *(ti-1)>=newT)
	{
		show(20,("        Imposible (hard constraint for activity, t=%d)\n", *(ti-1)));
		return *(ti-1)+1;
	}

	// check if the needed resource can be used
	FOREACH(r, act.requiredResources)
	{
		int modif = 7;
		FOR(t, newT, newT+len-1+(breakReq?ps.resources[r].breakLen:0))
		{
			if (t>=newT+len) modif = 3;
			if (FSResUsed[r][t]&modif WEEKS_ONLY( &(act.weekFlag|IMPOSSIBLE) ))
			{
				show(20,("        Imposible (needed resource %d [%s] can not be used, t=%d)\n", RES(r), t));
				show(30,("          Because FSResUsed[%d [%s]][%d]==%d, modif==%d\n", RES(r), t, FSResUsed[r][t], modif));
				return t+1;
			}
			//show(30, ("%d %d %d\n", t, newT+len-1, modif));
		}
	}
	
	// check if at least one resource from each group can be used
	REPV(gi, act.requiredGroups)
	{
		int g = act.requiredGroups[gi], minT = INT_MAX;
		REPV(ri, ps.groups[g].resources)
		{
			int r = ps.groups[g].resources[ri];
			if (resScore[gi][ri]>=UNASSIGNED_SCORE)
			{
				updateMin(minT, min(newT, time_sel[a])+1);
				continue;
			}
			int modif = 7;
			FOR(t, newT, newT+len-1+(breakReq?ps.resources[r].breakLen:0))
			{
				if (FSResUsed[r][t]&modif WEEKS_ONLY( &(act.weekFlag|IMPOSSIBLE) ))
				{
					//show(20,("        (Resource %d [%s] can not be used at time %d)\n", RES(r), t));
					updateMin(minT, t+1);
					goto badres;
				}
				if (t==newT+len-1) modif = 3;
			}
			goto goodres;
		badres:
			continue;	
		}
		show(20,("        Imposible (none of group resources can be used, t=%d)\n", minT-1));
		return minT;
	goodres:
		continue;
	}
	ok = 1;
	FSInsertAct(a, newT, len, breakReq, resScore);
	return 0;
}


int MySpace::FSTryMove(int a, int newT, int &curLength, bool &isBreak, int &ok, ResScore &resScore) 
// Try to Move activity a to time newT. 
// Typicaly newT is current start time +1, but other moves are also possible
// In curLength (before and after) we remember part of length of the activity 
//    which is allocated (this part is counted to score)
// In isBreak (before and after) we remember if the break after the activity
//    is currently allocated or not.
// Start time of the activity (before and after) is remembered in time_sel[a].
// This function also selects the best choice of group resources, in FSgrsel[].
// Function returns next t to check. It can be newT+1 or somthing bigger when 
//    we are sure that earlier value may be skipped.
// Incomming value of ok doesn't matter, at the end it says if the activity is 
//    correctly allocated.
// myScore and resScore will change apropiately, but changes of myScore 
//    doesn't take into account score from soft constraints of group resources.
//    These changes should be done later manually using resScore.
{
	CREF(act, ps.activities[a]);
	int newLen = act.length;
	REF(oldLen, curLength); // reference !!
	if (newT<0) newLen = 0;
	show(20,("      Trying to move activity %d [%s] from (%d-%d%s) to (%d-%d+break)\n", 
		ACT(a), time_sel[a], time_sel[a]+oldLen-1, isBreak?"+break":"", newT, newT+newLen-1));

	if (newLen && (newT+newLen-1)/ps.slotsPerDay!=newT/ps.slotsPerDay)
	// activity breaks end of day
	{
		ok = 0;
		return (newT+newLen-1)/ps.slotsPerDay*ps.slotsPerDay;
	}
	REF(oldT, time_sel[a]); // reference !!

	if (oldT<0)
		myScore -= UNASSIGNED_SCORE;

	// If the initial and the requested positions overlap we will skip 
	// changing anything in the overlaping part.
	
	if (oldLen==0)
		oldT = newT;
	
	// Remove the part of the old activity before newT
	if (oldT<newT)
	{
		int rem = min(oldLen, newT-oldT);
		FSRemoveAct(a, oldT, rem, isBreak&&rem==oldLen, resScore);
		if (rem==oldLen)
			isBreak = false;
		oldT = newT;
		oldLen -= rem;
	}

	// Remove the part of the old activity after newT+newLen-1
	if (oldT+oldLen>newT+newLen)
	{
		int rem = min(oldLen, oldT+oldLen-newT-newLen);
		FSRemoveAct(a, max(newT+newLen, oldT), rem, isBreak, resScore);
		oldLen -= rem;
		isBreak = false;
	}
	
	// Remove the break inside the new activity.
	if (isBreak && oldT+oldLen!=newT+newLen)
	{
		FSRemoveAct(a, oldT+oldLen, 0, true, resScore);
		isBreak = false;
	}

	if (oldLen==0)
		oldT = newT;

	// Now we have assigned only the overlapping part of activity.

	if (oldT<0)
		myScore += UNASSIGNED_SCORE;

	// Insert the part of the new activity after the old one
	if (newT+newLen>oldT+oldLen)
	{
		assert(newT+newLen-oldT-oldLen<=newLen);
		int next = FSInsertIfPossible(a, oldT+oldLen, newT+newLen-oldT-oldLen, !isBreak, ok, resScore); // ok is returned here
		if (!ok) return next;
		isBreak = true;
		oldLen = newT+newLen-oldT;
	}
	
	// Insert the part of the new activity before the old one
	if (newT<oldT)
	{
		assert(oldT-newT+oldLen==newLen);
		int next = FSInsertIfPossible(a, newT, oldT-newT, false, ok, resScore); // ok is returned here
		if (!ok) return next;
		oldLen = newLen;
		oldT = newT;
	}

	ok = 1;
	if (newT<0) 
		return 0;
	
	if (!isBreak)
	{
		int next = FSInsertIfPossible(a, oldT+oldLen, 0, true, ok, resScore); // ok is returned here
		if (!ok) return next;
		isBreak = true;
	}
		
	// for every group select best resource
	REPV(gi, act.requiredGroups)
	{
		int g = act.requiredGroups[gi], best = 0;
		REPV(ri, ps.groups[g].resources)
			if (resScore[gi][ri]<resScore[gi][best])
				best = ri;
		FSgrsel[act.grsel+gi] = ps.groups[g].resources[best];
	}

	return newT+1;
}

void MySpace::FSCreateResScore(int a, int curLength, ResScore &resScore)
// Creates resScore structure for activity a. 
// If currently the activity is used then curLength is length of the activity
// else curLength = 0.
{
	CREF(act, ps.activities[a]);
	resScore.begins = new int[size(act.requiredGroups)];
	int b = 0;
	REPV(gi, act.requiredGroups)
	{
		resScore.begins[gi] = b;
		b += size(ps.groups[act.requiredGroups[gi]].resources);
	}
	resScore.array = new int[b];
	
	// The special trick: I remove the activity, I zero resScore and 
	// then I insert the removed activity - resScore will be counted.
	// I have to remember current score because it will change incorrectly
	// during the first removing (when resScore is yet incorrect).
	int save = myScore;
	if (curLength>0)
	{
		firstTime = 1;
		FSRemoveAct(a, time_sel[a], curLength, false, resScore);
		firstTime = 0;
	}
	memset(resScore.array, 0, sizeof(resScore.array[0])*b);
	if (curLength>0)
		FSInsertAct(a, time_sel[a], curLength, false, resScore);
	myScore = save;
	show(20,("      resScore is now calculated, I start shufling\n"));
}

void MySpace::FSCreateResUsed()
/* Creates FSResUsed structure. */
{
	if (FSResUsed.empty())
	{
		FSResUsed.resize(size(ps.resources));
		FOREACH(a, FSResUsed)
			a.resize(ps.noOfSlots, 0);
	}
	FOREACH(a, FSResUsed)
		memset(&*a.begin(), 0, sizeof(FSResUsed[0][0])*ps.noOfSlots);
	REPV(a, ps.activities)
	{
		if (!used[a]) continue;
		CREF(act, ps.activities[a]);
		int rrsize = size(act.requiredResources);
		REP(ri, rrsize+size(act.requiredGroups))
		{
			int r = ri<rrsize ? act.requiredResources[ri] : FSgrsel[act.grsel+ri-rrsize];
			FOR(t, time_sel[a], time_sel[a]+act.length-1+ps.resources[r].breakLen)
				FSResUsed[r][t] WEEKS_ONLY2( |= act.weekFlag, |= 1);
		}
	}
	REPV(r, ps.resources)
		FOREACH(t, ps.resources[r].timePreferences.hard)
			FSResUsed[r][t] |= IMPOSSIBLE;
}

void MySpace::FastShufle()
{
	show(20,("Fast shufle\n"));

	CONT_ONLY(
	WEEKS_ONLY( REP(i, 2) 
	{)
		FScont_start WEEKS_ONLY([i]) = cont_start WEEKS_ONLY([i]);
		FScont_stop WEEKS_ONLY([i]) = cont_stop  WEEKS_ONLY([i]);
	WEEKS_ONLY( } )
	)
	FSdaysSel = daysSel;
	FSdaysNo = daysNo;

	FSCreateResUsed();
	for (;;)
	{
		int prevScore = myScore;
		// prevScore is a score at the begining of a pass
		// it is used only at the very end - when I tryed to move all activities:
		//  - if I got the same score I finish
		//  - if I got better score I try again
		
		show(20,("  Starting new pass (score = %d)\n", myScore));
		REPV(a, ps.activities)
		{
			//DEBUG_ONLY(CheckCurrentSolution();)
			CREF(act, ps.activities[a]); // <-- I'm moving this activity
			show(20,("    Trying with activity %d [%s] (time = %d, length = %d)\n", ACT(a), time_sel[a], act.length));
			int best = INT_MAX,	// best score reached by moving act
			    bestT = -1, 	// start time of act giving best score
			    curLength = time_sel[a]<0 ? 0 : act.length;
				// how long part of the activity is currently allocated
			bool isBreak = curLength;
				// is the break currently allocated or not
			
			vector<bool> allowed(ps.noOfSlots, true);
				// timeslots allowed by MaxDays and dependences
			
			// Remove the activity from 'FSdaysSel' and 'FSdaysNo'. (0)
			// Apply MaxDays constraint to 'allowed'.
			FOREACH(r, act.requiredResources)
			{
				CREF(res, ps.resources[r]);
				if (res.mdres<0) continue;
				int day = time_sel[a]/ps.slotsPerDay;
				if (curLength>0)
				{
					FSdaysSel[res.mdres][day] -= act.mdLength;
					if (FSdaysSel[res.mdres][day]==0)
						--FSdaysNo[res.mdres];
				}
				if (FSdaysNo[res.mdres]==res.maxDays)
					REP(d2, ps.daysPerWeek)
						if (FSdaysSel[res.mdres][d2]==0)
						{
							show(20,("      Day %d disallowed\n", d2));
							FOR(t, d2*ps.slotsPerDay, (d2+1)*ps.slotsPerDay-1)
								allowed[t] = false;
						}
				if (res.balancedScore>0)
					myScore -= CalculateBalancedScore(a, res, day, FSdaysSel);
			}
			
			// Apply dependences to 'allowed'.
			int from = 0, to = ps.noOfSlots-1;
			FOREACH(d, act.inDependences)
			{
				CREF(dep, ps.dependences[d]);
				int a2 = dep.firstAct^dep.secondAct^a;
				if (time_sel[a2]<0)
					continue;
				//show(30,("Dependence %d with %d: ", d, a2));
				switch (dep.relation)
				{
				case PS::Dependence::closelyBefore:
					updateMax(from, time_sel[a2]/ps.slotsPerDay*ps.slotsPerDay);
					updateMin(to, from+ps.slotsPerDay-1);
					// no break
				case PS::Dependence::before:
					if (a==dep.firstAct)
						updateMin(to, time_sel[a2]-curLength);
						//show(30, ("< %d\n", time_sel[a2]-curLength));
					else
						updateMax(from, time_sel[a2]+ps.activities[a2].length);
						//show(30, ("> %d\n", time_sel[a2]+ps.activities[a2].length));
					break;
				case PS::Dependence::concurrently:
					if (a==dep.firstAct)
					{
						updateMax(from, time_sel[a2]+ps.activities[a2].length-curLength);
						updateMin(to, time_sel[a2]);
					}
					else
					{
						updateMax(from, time_sel[a2]);
						updateMin(to, time_sel[a2]+ps.activities[a2].length-curLength);
					}
					break;
				default:
					assert(false);
				}
			}
			FOR(t, 0, min(from-1, ps.noOfSlots-1))
				allowed[t] = false;
			FOR(t, max(0, to+1), ps.noOfSlots-1)
				allowed[t] = false;
			
			// Marking group resources used by this activity as unused in FSResUsed (1)
			REPV(gi, act.requiredGroups)
			{
				int r = FSgrsel[act.grsel+gi];
				FOR(t, time_sel[a], time_sel[a]+curLength-1+(isBreak?ps.resources[r].breakLen:0))
					FSResUsed[r][t] &= ~WEEKS_ONLY2(act.weekFlag, 1); 
				if (20<=verboseMode) {
					int ri_ok = -1;
					REPV(ri, ps.groups[act.requiredGroups[gi]].resources)
						if (ps.groups[act.requiredGroups[gi]].resources[ri]==FSgrsel[act.grsel+gi])
						{
							assert(ri_ok==-1);
							ri_ok = ri;
						}
					if (curLength)
					{
						assert(ri_ok>=0);
						show(20,("    Selected resource: %d [%s] (index in group: %d)\n", 
							RES(FSgrsel[act.grsel+gi]), ri_ok));
					}
				}
			}

			vector<int> balres;	
				// Resources used by this activity with
				// Balanced constraint.
			FOREACH(r, act.requiredResources)
				if (ps.resources[r].balancedScore>0)
					balres.push_back(r);

			
			DEBUG_ONLY( int t0 = time_sel[a]; bool t0avail = false; )
			ResScore resScore;
			FSCreateResScore(a, curLength, resScore);
			FSUpdateMyScoreFromRes(a, -1, resScore);
			for (int t = 0; t<ps.noOfSlots; )
			{
				if (!allowed[t])
				{
					show(20,("      Time = %d -> skip\n", t));
					++t;
					continue;
				}
				int st = t, ok;
				t = FSTryMove(a, t, curLength, isBreak, ok, resScore); // ok is returned here
				if (ok)
				{
					DEBUG_ONLY( if (st==t0) t0avail = true; )
					FSUpdateMyScoreFromRes(a, +1, resScore);
					int add = 0;
					FOREACH(r, balres)
						add += CalculateBalancedScore(a, ps.resources[r], st/ps.slotsPerDay, FSdaysSel);
					if (myScore+add<best)
					{
						bestT = st;
						best = myScore+add;
					}
					show(20,("      Time = %d -> ok, score = %d\n", st, myScore+add));
					FSUpdateMyScoreFromRes(a, -1, resScore);
				}
				else
					show(20,("      Time = %d -> impossible\n", st));
			}
			assert(t0avail||t0<0);
			int ok;
			FSTryMove(a, bestT, curLength, isBreak, ok, resScore);
			assert(curLength==(time_sel[a]<0 ? 0 : act.length));
			FSUpdateMyScoreFromRes(a, +1, resScore);

			// Undoing (1)
			REPV(gi, act.requiredGroups)
			{
				int r = FSgrsel[act.grsel+gi];
				FOR(t, time_sel[a], time_sel[a]+curLength-1+(isBreak?ps.resources[r].breakLen:0))
					FSResUsed[r][t] WEEKS_ONLY2( |= act.weekFlag, |= 1); 
			}

			// Undoing (0)
			if (curLength>0)
			{
				FOREACH(r, act.requiredResources)
				{
					CREF(res, ps.resources[r]);
					if (res.mdres<0) continue;
					int day = time_sel[a]/ps.slotsPerDay;
					if (res.balancedScore>0)
						myScore += CalculateBalancedScore(a, res, day, FSdaysSel);
					int save = FSdaysSel[res.mdres][day];
					FSdaysSel[res.mdres][day] += act.mdLength;
					if (save==0)
						++FSdaysNo[res.mdres];
				}
			}
			show(20,("    Selecting %d, score is %d (should be %d)\n", bestT, myScore, best));
			assert(!(bestT>=0 && best!=myScore));
			assert(myScore<=prevScore);
		}
		if (prevScore==myScore) break;
	}
	show(10,("Fast shufle gives score %d\n", myScore));
}

} // namespace Spaces
