/*
 *  Author: Pawel Parys <parys@mimuw.edu.pl> (2003-2004)
 *
 *  For licence see doc/LICENCE.
 */

#if 0

#include "globalprop.h"
#include "space.h"
#include "problemspec.h"
#include "defines.h"
#include "xxx.h"
#include <algorithm>
#include <cassert>

/* See globalprop.h for comments. */

namespace Propagators
{

using namespace Spaces;

AllDisjointPropagator::AllDisjointPropagator(Spaces::Space &sp, const vector<pair<int, int> > &activities, int globNo)
	: activities(activities), globNo(globNo) 
{ 
//	Initialize(sp);
//	FOREACH(a, activities)
//		InitializeVar(sp, a->first);
}

void insert(int num, int prev, vector<vector<int> > &back, vector<int> &layers, 
	int curlayer, vector<int> &current)
{
	if (layers[num]!=-1 && layers[num]!=curlayer)
		return;
	if (layers[num]==-1)
	{
		current.push_back(num);
		layers[num] = curlayer;
	}
	back[num].push_back(prev);
}

bool find_path(int num, vector<vector<int> > &back, vector<int> &layers, vector<int> &matching)
{
	if (layers[num]<0) return false;
	layers[num] = -1;
	FOREACH(num2i, back[num])
	{
		if (layers[*num2i]<0) break;
		assert((layers[*num2i]==0 && size(back[*num2i])==0) || size(back[*num2i])==1);
		layers[*num2i] = -1;
		if (back[*num2i].empty() || find_path(back[*num2i][0], back, layers, matching))
		{
			matching[*num2i] = num;
			matching[num] = *num2i;
			return true;
		}
	}
	return false;
}

void DFS1(int v, vector<vector<int> > &arcs, vector<int> &tab, vector<int> &order)
{
	if (tab[v]>=0) return;
	tab[v] = 1;
	FOREACH(ai, arcs[v])
		DFS1(*ai, arcs, tab, order);
	order.push_back(v);
}

void DFS2(int v, vector<vector<int> > &arcs, vector<int> &tab, int component)
{
	if (tab[v]>=0) return;
	tab[v] = component;
	FOREACH(ai, arcs[v])
		DFS2(*ai, arcs, tab, component);
}

void AllDisjointPropagator::Propagate(Space &sp, int myNo)
{
	show(11, ("{AllDisjointPropagator no %d is working}\n", myNo));
//	Actualize(sp, myNo);
	int N0 = size(sp.time[0].possible);	// No of timeslots.
	int match_size = 0;	// Current number of edges in matching.
	bool have_old = true;

	/* If first call - generate entry in matching array. */
	if (globNo<0)
	{
		globNo = size(sp.matching);
		sp.matching.resize(globNo+1);
		int N1 = 0;
		FOREACH(ai, activities)
			N1 += ai->second;
		sp.matching[globNo].resize(N0+N1, -1);
		have_old = false;
	}
	
	vector<int> &matching = sp.matching[globNo];
	int N = size(matching);
	int N1 = N-N0;	// no of activity nodes
	if (have_old)
		match_size = N1;
	
	/* Generate edges array. The vertices are sorted. */
	show(11, ("Generating graph\n"));
	vector<vector<int> > edges(N);
	int pos0 = N0;
	FOREACH(ai, activities)
	{
		show(11,("Activity %d[%s] (len=%d): ", ACT(ai->first), ai->second));
		const Space::Time &time = sp.time[ai->first];
		int left = ai->second;
		FOR(t, time.min, time.max+ai->second-1)
		{
			if (time.possible[t] && left!=ai->second-1)
				show(11,("%d-", t));
			if (!time.possible[t] && left==ai->second-1)
				show(11,("%d ", t-1));
			if (time.possible[t]) left = ai->second;
			if (left>0)
			{
				edges[pos0+t%ai->second].push_back(t);
				edges[t].push_back(pos0+t%ai->second);
			}
			--left;
		}
		if (left==ai->second-1)
			show(11,("%d", time.max+ai->second-1));
		show(11,("\n"));
/*		REP(t, N0)
			if (!edges[t].empty() && edges[t][size(edges[t])-1]>=pos0)
				show(11,("%d ", t));
		show(11,("\n")); */
		pos0 +=  ai->second;
	}
	
	/* Remove a pair from matching if there is no such edge (it was
	 * removed since last call). */
	if (have_old)
		FOR(v1, N0, N-1)
			if (matching[v1]>=0 && !binary_search(ALL(edges[v1]), matching[v1]))
			{
				matching[matching[v1]] = -1;
				matching[v1] = -1;
				--match_size;
			}
	
	/* Find the maximal matching (Hopcroft + Karp). */
	show(11, ("Calculate maximal matching\n"));
	while (match_size<N1)
	{
		/* Look for extending paths. Maximal set of disjoint
		 * extenting paths having minimal length will be found. */
		
		vector<vector<int> > back(N);	// Previous vertex on path,		
		vector<int> layers(N, -1);	// Layer of element (-1 =
						// unvisited).
		vector<int> current[2];	// List of vertices on current
					// level.
		
		/* Find first layer of vertices. */
		FOR(v1, N0, N-1)
		 	if (matching[v1]<0)
		 	{
		 		layers[v1] = 0;
		 		current[0].push_back(v1);
		 	}
		 
		/* And all next leyers. */
		for(int curlayer = 1; ; curlayer += 2)
		{
			int finish = 0;
			current[1].clear();
			FOREACH(v1i, current[0])
				FOREACH(v2i, edges[*v1i])
				{
		 			insert(*v2i, *v1i, back, layers, curlayer, current[1]);
		 			if (matching[*v2i]<0)
		 				finish = 1;
		 		}
		 	if (finish) break;	// Free vertex reached.
		 	if (current[0].empty())
		 	{
		 		show(11,("Throw\n"));
		 		throw -1;
		 	}
			
		 	current[0].clear();
			FOREACH(v1i, current[1])
				insert(matching[*v1i], *v1i, back, layers, curlayer+1, current[0]);
		 	if (current[1].empty())
		 	{
		 		show(11,("Throw\n"));
		 		throw -1;
		 	}
		}

		/* Find disjoint paths in the tree. Apply changes in
		 * matching according to found paths. */
		FOREACH(v0, current[1])
			if (matching[*v0]<0)
			{
				assert(layers[*v0]>0);
				if (find_path(*v0, back, layers, matching))
					++match_size;
			}
	}
	
	/* Generate new edges: (Let X be activity nodes, Y - timeslots.)
	 * Edges in matching will be from Y to X. Other edges from original
	 * graph will be from X to Y. Additionaly there will be arcs from
	 * every free node of Y to every matched node of Y. */
	show(11, ("Generate new graph\n"));
	vector<vector<int> > arcs(N), arcs_back(N);
		// New graph (forward and backward).
	FOR(v1, N0, N-1)
	{
		assert(matching[v1]>=0);
		show(11,("matching[%d]=%d\n", v1, matching[v1]));
		arcs[matching[v1]].push_back(v1);
		arcs_back[v1].push_back(matching[v1]);
		FOREACH(v0i, edges[v1])
			if (*v0i!=matching[v1])
			{
				arcs[v1].push_back(*v0i);
				arcs_back[*v0i].push_back(v1);
			}
		REP(v0, N0)
			if (matching[v0]<0 && !edges[v0].empty())
			{
				arcs[v0].push_back(matching[v1]);
				arcs_back[matching[v1]].push_back(v0);
			}
	}
	
	/* Find strongly connected components. */
	show(11, ("Strongly connected components\n"));
	vector<int> order;	// List of vertices in DFS post-order.
	vector<int> tab(N, -1);
	REP(v, N)
		DFS1(v, arcs, tab, order);
	assert(size(order)==N);
	vector<int> component(N, -1);	// Number of component for each activity.
	FOREACHBACK(vi, order)
		DFS2(*vi, arcs_back, component, *vi);
	
	/* Remove edges not in matching between different stronly connected
	 * components. Convert edges into possible times. */
	show(11, ("Postprocessing\n"));
	pos0 = N0;
	FOREACH(ai, activities)
	{
		Space::Time &time = sp.time[ai->first];
		int lock = 0;
		bool changed = false;
		FORD(t, time.max+ai->second-1, time.min)
		{
			int v = pos0+t%ai->second;
			if (edges[v].empty() || edges[v][size(edges[v])-1]!=t)
				lock = ai->second;
			else
			{
				edges[v].resize(size(edges[v])-1);
				if (matching[v]!=t && component[v]!=component[t])
					lock = ai->second;
			}
			if (lock>0 && time.possible[t])
			{
				time.possible[t] = false;
				changed = true;
		 		show(11,("Removed %d for %d [%s]\n", t, ACT(ai->first)));
			}
			--lock;
		}
		pos0 += ai->second;
		if (changed)
		{
			sp.UpdateMinMax(ai->first);
			sp.VariableChanged(ai->first);
		}
	}
	show(11, ("Return\n"));
}


} // namespace Propagators

#endif
