/*
 *  Author: Pawel Parys <parys@mimuw.edu.pl> (2003-2004)
 *
 *  For licence see doc/LICENCE.
 */

#ifndef INPUTPARSER_MY_H
#define INPUTPARSER_MY_H

#include <string>
#include <cstdio>
using namespace std;

/* Parser for XML files. Correspods to inputparser.y. */

/* The following functions shows error message 's' and exits program.
 * 's' should not have trailing '\n', it's added automaticaly. The
 * 'yyerror_arg' function can take additional arguments (like 'printf'). 
 * 'yyerror_line' additionaly displays line number in which the error 
 * occurs. */

int yyerror(const char *s);
void yyerror_arg(const char *s, ...);
void yyerror_arg(int line, const char *s, ...);

extern int yyline;	// Current line number in lexer.

extern string openedFile;	// Currently opened file.

namespace LexYacc
{
/* 'addstring' remembers the arguments and returns its number. Then you can
 * get it back calling 'getstring' with this number. Leading and trailing
 * white spaces are removed. The functions can be used only while parsing.
 * They are used to pass strings between lexer and parser (direct
 * communication is not very easy). */
int addstring(const string &str);
const string &getstring(int nr);
}

namespace XML { class Elements; }

void parseFile(FILE *file, XML::Elements &result);
/* Parse XML file, 'result' should be empty. */

#endif
