/*
 *  Author: Pawel Parys <parys@mimuw.edu.pl> (2003-2004)
 *
 *  For licence see doc/LICENCE.
 */

#ifndef __MULTIPROC_H
#define __MULTIPROC_H

#include "defines.h"

/* This module implements multiprocessing, allocates memory for
 * 'bestSolution', 'bestSugestion', 'prototypeSolution' and installs signal
 * handling. For more information see multiproc.cc. 
 *
 * IMPORTANT: Call InitializeMultiproc even when multiprocessing is turned
 * off. */

namespace Analyser { class ProblemSpecification; }
namespace Spaces { class MySpace; }

extern int noOfProcesses;	// Number of child processes, 0 when
				// multiprocessing turned off.
extern int fastStop;	// Set to 1 after SIGTERM or SIGINT signal. 

/* To start the module call 'initializeMultiproc', at the end call
 * 'closeMultiproc'. */
void initializeMultiproc(const Analyser::ProblemSpecification &ps, Spaces::MySpace &space);
void closeMultiproc();

MULTI_ONLY( void tryChangeMultiproc(const Analyser::ProblemSpecification &ps, int act); )
/* Called by main process to give a work to one of subprocesses. Call it
 * only when multiprocessing turned on, instead of tryChange. */

/* Use the following two functions to lock unlock access to bestSolution and
 * bestSugestion. (Reading from bestScore is done without locking, assuming
 * it atomic operation.) */
MULTI_ONLY( void lockSolution(int readonly=1); )
MULTI_ONLY( void unlockSolution(int readonly=1); )

#ifdef _WIN32
/* Says if stdin is used for reading problem specification or something. If
 * not set, then any characters on stdin will terminate the program (only
 * under Windows). */
extern bool stdinAlreadyUsed;
#endif

#endif
