/*
 *  Author: Pawel Parys <parys@mimuw.edu.pl> (2003-2004)
 *
 *  For licence see doc/LICENCE.
 */

#include "propagators.h"
//#include "globalprop.h"	//--++
#include "myspace.h"
#include "space.h"
#include "tools.h"
#include "problemspec.h"
#include "multiproc.h"
#include "solution.h"
#include <cstdio>
#include <vector>
#include <set>
#include <cassert>
#include "xxx.h"

using namespace std;

using namespace Analyser;
using namespace Propagators;

DEBUG_ONLY(
WEEKS_ONLY(
bool operator==(const int3tab &a, const int3tab &b){			//!!!!!+++--++
	return a.tab[0]==b.tab[0]&&a.tab[1]==b.tab[1]&&a.tab[2]==b.tab[2];
}
))

namespace Spaces
{
DEBUG_ONLY(
bool operator==(const MySpace::Info &a, const MySpace::Info &b){//!!!!!+++--++
	return a.act==b.act && a.time==b.time && a.res==b.res; 
})

int MySpace::searchSugested;

inline int MySpace::Random()
{	
	srand(randseed);
	return randseed = rand();
}

void MySpace::InitializeTime(int actNo)
{
	CREF(act, ps.activities[actNo]);
	REF(t, time[actNo]);
	t.Initialize(ps.noOfSlots);
	changedVar.push(actNo);
	REP(a, ps.daysPerWeek)
		Exclude(actNo, (a+1)*ps.slotsPerDay-act.length+1, (a+1)*ps.slotsPerDay-1);
	FOREACH(ti, act.timePreferences.hard)
		Exclude(actNo, ti-act.length+1, ti);
}

MySpace::MySpace() : Space(size(ps.activities)), myScore(0), randseed(rand())
{
	// initialize empty structs
	int noOfAct = size(time);
	waiting.resize(noOfWaiting = noOfAct);
	used.resize(noOfAct, false);
	time_sel.resize(noOfAct);
	REP(a, noOfAct)
	{
		try
		{
			InitializeTime(a);
		}
		catch(...)
		{
			yyerror_arg("Activity %d [%s] do not have any timeslots available", ACT(a));
		}
		waiting[a] = a;
	}
	FORD(a, noOfAct-1, 0)
		swap(waiting[a], waiting[rand()%(a+1)]);
	daysSel.resize(ps.mdresNo);
	daysNo.resize(ps.mdresNo);
	FOREACH(d, daysSel)
		d.resize(ps.daysPerWeek);
	isGrresUsed.resize(ps.grresNo);
	FOREACH(a, isGrresUsed)
		a.resize(ps.noOfSlots, 0);
	availableRes.resize(size(ps.groups));
	REPV(a, ps.groups)
	{
		CREF(vec, ps.groups[a].resources);
		if (20<=verboseMode)
		{
			fprintf(stderr2, "Group %d has %d resources (", a, size(vec));
			REPV(z, vec)
				fprintf(stderr2, "%d%s", vec[z], z==size(vec)-1?")\n":", ");
			fflush(stderr2);
		}
		availableRes[a].resize(ps.noOfSlots, WEEKS_ONLY2(int3tab(0, 0, size(vec)), size(vec)));
	}
	grsel.resize(ps.grselNo, -1);
	info.push(Info());
	CONT_ONLY(
	WEEKS_ONLY( REP(w, 2) )
	{
		cont_start WEEKS_ONLY([w]) .resize(ps.contresNo);
		cont_stop WEEKS_ONLY([w]) .resize(ps.contresNo);
		REP(a, ps.contresNo)
		{
			cont_start WEEKS_ONLY([w]) [a].resize(ps.daysPerWeek, ps.noOfSlots);
			cont_stop WEEKS_ONLY([w]) [a].resize(ps.daysPerWeek, 0);
		}
	}
	)
	
	// time preferences -> availablity of resources
	REPV(r, ps.resources)
	{
		CREF(res, ps.resources[r]);
		if (res.grres>=0)
			FOREACH(ti, res.timePreferences.hard)
				RemoveResource(r, ti, ti WEEKS_ARG(ALL_WEEKS));
	}

	// resources conflicts -> disjoint activities
	REPV(a, ps.activities)
	{
		CREF(act, ps.activities[a]);
		REPV(a2i, act.conflictsWith)
		{
			int a2 = act.conflictsWith[a2i].first;
			int bl = act.conflictsWith[a2i].second;
			if (a<a2)
			{
				show(20,("Activity %d [%s] conflicts with %d [%s] (break=%d)\n", ACT(a), ACT(a2), bl));
				new DisjointPropagator(*this, a, act.length+bl, a2, ps.activities[a2].length+bl);
			}
		}
	}
	
/*	FOREACH(ri, ps.resources)			//--++
		if (size(ri.inActivitiesRe)>=3)
		{
			vector<pair<int, int> > dis_act;
			FOREACH(ai, ri.inActivitiesRe)
				dis_act.push_back(make_pair(ai, ps.activities[ai].length+ri.breakLen));
			globalprop.push_back(new AllDisjointPropagator(*this, dis_act));
		}
	*/
	

	// dependences
	FOREACH(dep, ps.dependences)
	{
		if (dep.score>=0)
			continue;
		switch (dep.relation)
		{
		case PS::Dependence::before:
			new BeforePropagator(*this, dep.firstAct, ps.activities[dep.firstAct].length, dep.secondAct);
			break;
		case PS::Dependence::closelyBefore:
			new BeforePropagator(*this, dep.firstAct, ps.activities[dep.firstAct].length, dep.secondAct);
			new SameDayPropagator(*this, dep.firstAct, dep.secondAct, ps.slotsPerDay);
			break;
		case PS::Dependence::concurrently:
			new ConcurrentlyPropagator(*this, dep.firstAct, dep.secondAct, ps.activities[dep.firstAct].length-ps.activities[dep.secondAct].length);
			break;
		default:
			break;
		}
	}
}

void MySpace::FindAndChooseBestAct()
{
	info.top().act = waiting[--noOfWaiting];
}

void MySpace::FindBestTime(int a)
{
	vector<int> best;
	int score = INF;
	CREF(time, time[info.top().act]);
	CREF(act, ps.activities[a]);
	vector<int> balres CONT_ARG(cres);
	FOREACH(r, act.requiredResources)
	{
		CONT_ONLY(
		if (ps.resources[r].contres>=0)
			cres.push_back(r);
		)
		if (ps.resources[r].balancedScore>0)
			balres.push_back(r);
	}
	FOR(t, time.min, time.max)
		if (time.possible[t])
		{
			int sc = act.timePreferences.compressedSoft[t];
			int day = t/ps.slotsPerDay;
			CONT_ONLY(
			FOREACH(r, cres)
			{
				REF(res, ps.resources[r]);
				int inc = 0;
				WEEKS_ONLY2(
				if ((act.weekFlag & EVEN_WEEKS) && cont_stop[0][res.contres][day])
					inc = max(0, cont_start[0][res.contres][day]-t)+
						max(0, t+act.length-1-cont_stop[0][res.contres][day]);
				if ((act.weekFlag & ODD_WEEKS) && cont_stop[1][res.contres][day])
					inc += max(0, cont_start[1][res.contres][day]-t)+
						max(0, t+act.length-1-cont_stop[1][res.contres][day]);
				sc += inc*(res.continuousScore/2);
				,
				if (cont_stop[res.contres][day])
					inc = max(0, cont_start[res.contres][day]-t)+
						max(0, t+act.length-1-cont_stop[res.contres][day]);
				sc += inc*res.continuousScore;
				)
			})
			FOREACH(r, balres)
				sc += CalculateBalancedScore(a, ps.resources[r], day, daysSel);
			if (sc<score)
			{
				score = sc;
				best.clear();
			}
			if (sc==score)
				best.push_back(t);
		}
	info.top().time = best[Random()%size(best)];
}

void MySpace::FindBestResource()
{
	int a = info.top().act;
	CREF(act, ps.activities[a]);
	int selno = size(info.top().res)-1;
	int g = act.requiredGroups[selno];
	CREF(rlist, ps.groups[g].resources);
	FOREACH(ri, rlist)
	{
		int grres = ps.resources[ri].grres;
		int bl = ps.resources[ri].breakLen;
		FOR(t, time[a].min, time[a].min+act.length+bl-1)
			if (isGrresUsed[grres][t] WEEKS_ONLY(&act.weekFlag))
			{
				show(20,("[Resource %d [%s] (grres = %d) unavailable at %d (needed by %d [%s])]\n", 
					RES(ri), grres, t, ACT(a)));
				goto nonavail;
			}
		info.top().res.top().push_back(ri);
		return;
	nonavail:
		continue;
	}
	throw failno=a;
}

void MySpace::FindAllResources()
{
	int a = info.top().act;
	int selno = size(info.top().res)-1;
	CREF(act, ps.activities[a]);
	int g = act.requiredGroups[selno];
	CREF(rlist, ps.groups[g].resources);
	REF(possible, info.top().res.top());
	FOREACH(ri, rlist)
	{
		int grres = ps.resources[ri].grres;
		int bl = ps.resources[ri].breakLen;
		FOR(t, time[a].min, time[a].min+act.length+bl-1)
			if (isGrresUsed[grres][t] WEEKS_ONLY(&act.weekFlag))
				goto nonavail;
		if (ri == prototypeSolution.grsel[ps.activities[a].grsel+selno] && 
			!possible.empty())
		{
			possible.push_back(possible[0]);
			possible[0] = ri;
		}
		else
			possible.push_back(ri);
	nonavail:
		continue;
	}
	if (possible.empty())
		throw failno=a;
}

int MySpace::PrepareSelection()
{
	WaitStable();
	if (state!=normal) 
		return 0;
	if (!noOfWaiting)
	{
		Succed();
		return 0;
	}
	FindAndChooseBestAct();
	FindBestTime(info.top().act);
	return 2;
}

int MySpace::PrepareSelection_All()
{
	WaitStable();
	if (state!=normal) 
		return 0;
	if (AreWeSelectingTime())
	{
		if (!noOfWaiting)
		{
			Succed();
			return 0;
		}
		FindAndChooseBestAct();
		int c = 1;
		REP(t, ps.noOfSlots)
			if (time[info.top().act].possible[t])
				++c;
		return c;
	}
	else
	{
		try
		{
			FindAllResources();
		}
		catch(...)
		{
			Fail();
		}
		return size(info.top().res.top());
	}
}

void MySpace::DisjointWithActivities(const vector<int> &vec, int start, int stop WEEKS_ARG(int specialFlag))
/* 'vec' is vector of activities numbers. We say these the activities can
 * not use slots from 'start' to 'stop'. Depending on 'specialFlag' we take
 * into account only activities using: only odd (|1), only even (|2), both
 * (|4) weeks. (It is possible, that we want to remove only resources using
 * both weeks, but leave resources using only odd weeks and only even weeks
 * - when different resources are available in odd and even weeks). */
{
	FOREACH(a2i, vec)
		if (!used[a2i])
		{
			CREF(act2, ps.activities[a2i]);
			WEEKS_ONLY(
			  /* Is this the type of activity we want to remove? */
			  const int translate = act2.weekFlag==3 ? 4 : act2.weekFlag;
			  if (!(translate&specialFlag)) continue;
			)
			show(20,("Activity %d [%s] can not start between %d and %d\n", ACT(a2i), start-act2.length+1, stop));
			Exclude(a2i, start-act2.length+1, stop);
		}
}

void MySpace::RemoveResourceByAct(int a, int r)
/* We say that activity 'a' uses resource 'r'. Mark, that the resource is
 * not available while used by the activity (+break). Update 'daysSel' and
 * 'daysNo'. Disallow activities dependent from 'r' to use the timeslots.
 * Update score from 'Balanced' constraint. */
{
	show(20,("Activity %d [%s] needs resource %d [%s]\n", ACT(a), RES(r)));
	CREF(res, ps.resources[r]);
	CREF(act, ps.activities[a]);
	if (res.grres>=0)
	{
		int bl = res.breakLen;
		RemoveResource(r, time[a].min, time[a].min+act.length+bl-1 
			WEEKS_ARG(act.weekFlag));
	}
	if (res.mdres>=0)
	{
		int day = time[a].min/ps.slotsPerDay;
		if (res.balancedScore>0)
		{
			int bs = CalculateBalancedScore(a, res, day, daysSel);
			myScore += bs;
			if (bs>0) 
				show(25, ("Resource %d [%s] in day %d causes 'balanced' score %d\n", RES(r), day, bs/res.balancedScore));
		}
		int save = daysSel[res.mdres][day];
		daysSel[res.mdres][day] += act.mdLength;
		if (save==0 && res.maxDays==++daysNo[res.mdres])
			REP(d, ps.daysPerWeek)
				if (daysSel[res.mdres][d]==0)
					FOREACH(a, res.inActivitiesRe)
						Exclude(a, d*ps.slotsPerDay, (d+1)*ps.slotsPerDay-1);
	}
}

void MySpace::RemoveResource(int r, int from, int to WEEKS_ARG(int flag))
/* 'r' is number of resource, but it has to be group resource. We say that
 * this resource is not available from 'from' to 'to' in odd (flag=1), even
 * (flag=2) or both (flag=3) weeks. Modified structures: 'isGrresUsed',
 * 'availableRes'. Additionaly activities using this resource can not use
 * these timeslots. When it is last resource in group at given time, then
 * activities using the group can not use these timeslots. */
{
	show(20,("Resource %d [%s] unavailable from %d to %d" WEEKS_ONLY(" (flag=%d)") "\n", RES(r), from, to WEEKS_ARG(flag)));
	CREF(res, ps.resources[r]);
	int grres = res.grres;
	DisjointWithActivities(res.inActivitiesRe, from, to WEEKS_ARG(4|flag));
	
	FOR(t, from, to)
	{
		WEEKS_ONLY( const int prev = ((~isGrresUsed[grres][t])&3)-1; )
		isGrresUsed[grres][t] |= WEEKS_ONLY2(flag, 1);
		WEEKS_ONLY( const int cur = ((~isGrresUsed[grres][t])&3)-1; )

		/* 'prev' (and 'cur') says when the resource is available:
		 * in odd (0), even(1) (REVERSED!!!), both(2) weeks or never
		 * (-1) before (and after) the remove. */

		/* Now for each time and group change group data. */
		FOREACH(gi, res.inGroups)
		{
			REF(vec, availableRes[gi]);
			WEEKS_ONLY2( 
			  --vec[t][prev];
			  if (cur>=0) ++vec[t][cur];
			,
			  --vec[t];
			)
			show(20,("In group %d at time %d there are %d" WEEKS_ONLY(", %d, %d") " resources left\n", gi, t, 
				vec[t] WEEKS_ONLY([0]) WEEKS_ARG(vec[t][1]) WEEKS_ARG(vec[t][2])));
			if (!vec[t] WEEKS_ONLY([2]))
			/* There are no resources available (for both weeks). */
			{
				WEEKS_ONLY(
				const int specialFlag = 4 | (vec[t][0] ? 0 : 1) | (vec[t][1] ? 0 : 2);
				)
				/* 'specialFlag' says for which activieties
				 * the group will be not available: for
				 * these in only odd (|1), only even (|2) or
				 * both(|4) weeks. */
				
				show(20,("Group %d empty at time %d" WEEKS_ONLY(" (specialFlag=%d)") "\n", gi, t 
					WEEKS_ARG(specialFlag)));
				DisjointWithActivities(ps.groups[gi].activities, t, t WEEKS_ARG(specialFlag));
			}
		}
	}
}

void MySpace::UndoRemoveResourceByAct(int a, int r)
/* We say that activity 'a' no more uses resource 'r'. We mark, that the
 * resource is available again in the time when there was the activity
 * (+break). It corrects only 'isGrresUsed', 'availableRes', 'daysSel',
 * 'daysNo' structures, but do not change anything for activities dependent
 * from this resource. */
{
	CREF(res, ps.resources[r]);
	CREF(act, ps.activities[a]);
	if (res.grres>=0)
	{
		int bl = res.breakLen;
		UndoRemoveResource(r, time[a].min, time[a].min+act.length+bl-1 
			WEEKS_ARG(act.weekFlag));
	}
	if (res.mdres>=0)
	{
		int day = time[a].min/ps.slotsPerDay;
		daysSel[res.mdres][day] -= act.mdLength;
		if (daysSel[res.mdres][day]==0)
			--daysNo[res.mdres];
	}
}

void MySpace::UndoRemoveResource(int r, int from, int to WEEKS_ARG(int flag))
/* Undo the 'RemoveResource' operation. 'r' is number of resource, but it
 * has to be group resource. We say that this resource is again available from
 * 'from' to 'to' in odd (flag=1), even (flag=2) or both (flag=3) weeks.
 * It only modifies structures: 'isGrresUsed', 'availableRes', but it do not
 * change anything for activities dependent from this resource. */ 
{
	CREF(res, ps.resources[r]);
	int grres = res.grres;
	FOR(t, from, to)
	{
		WEEKS_ONLY( const int prev = ((~isGrresUsed[grres][t])&3)-1; )
		isGrresUsed[grres][t] &= ~WEEKS_ONLY2(flag, 1);
		WEEKS_ONLY( const int cur = ((~isGrresUsed[grres][t])&3)-1; )

		/* 'prev' (and 'cur') says when the resource is available:
		 * in odd (0), even(1) (REVERSED!!!), both(2) weeks or never
		 * (-1) before (and after) the undo. */

		/* Now for each time and group change group data. */
		FOREACH(gi, res.inGroups)
		{
			REF(vec, availableRes[gi]);
			WEEKS_ONLY2( 
			  if (prev>=0) ++vec[t][prev];
			  ++vec[t][cur];
			,
			  ++vec[t];
			)
		}
	}
}


inline void MySpace::ScoreFromSoftPreferences(int from, int to, const PS::TimePreferences &tp 
	WEEKS_ARG(int divide))
{
	FOR(a, from, to)
		myScore += tp.soft[a] WEEKS_ONLY(/divide);
	ScoreUpdated();
}

#if ENABLE_CONT
int MySpace::CalculateContinuousScore(const PS::Resource &res WEEKS_ARG(int which), 
	vector<vector<int> > cont_start WEEKS_ONLY([2]), vector<vector<int> > cont_stop WEEKS_ONLY([2]))
{
	int x = 0;
	CREF(start, cont_start WEEKS_ONLY([which]) [res.contres]);
	CREF(stop, cont_stop WEEKS_ONLY([which]) [res.contres]);
//	int days = 0;
	REP(d, ps.daysPerWeek)
		if (start[d]<=stop[d])
//		{
			x += stop[d]-start[d]+1;
//			++days;
//		}
	x -= res.contCount WEEKS_ONLY([which]);// - days*res.breakLen;
	if (x>0) 
	{
		show(20,(" %s.%d:%d(->%d)[", res.id.c_str(), res.contres, res.contCount WEEKS_ONLY([which]), x));
		REP(d, ps.daysPerWeek)
			show(20,("%d %d%s", start[d], stop[d], d==ps.daysPerWeek-1?"]\n":" "));
	}
	return x>0 ? x : 0;
}

inline void MySpace::UpdateContinuous(int from, int to, const PS::Resource &res WEEKS_ARG(int which))
{
	int score = res.continuousScore;
	myScore -= CalculateContinuousScore(res WEEKS_ARG(which), cont_start, cont_stop)*(score WEEKS_ONLY(/2));
	REF(start, cont_start WEEKS_ONLY([which]) [res.contres][from/ps.slotsPerDay]);
	REF(stop, cont_stop WEEKS_ONLY([which]) [res.contres][from/ps.slotsPerDay]);
	journal_cont.push(ContInfo(res.contres, from/ps.slotsPerDay WEEKS_ARG(which), start, stop));
	updateMin(start, from);
	updateMax(stop, to);
	myScore += CalculateContinuousScore(res WEEKS_ARG(which), cont_start, cont_stop)*(score WEEKS_ONLY(/2));
}	// ScoreUpdated() have to be called later
#endif

void MySpace::ChooseTime(int t)
{
	int a = info.top().act;
	journal_selection.push(make_pair(a, -1));
	show(19,("Activity %d [%s] starts at %d (bestSug: %d)\n", ACT(a), t, prototypeSolution.time[a]));
	SetVar(a, t);
	CREF(act, ps.activities[a]);
	used[a] = true;
	FOREACH(ri, act.requiredResources)
	{
		RemoveResourceByAct(a, ri);
		CONT_ONLY(
		CREF(res, ps.resources[ri]);
		if (res.contres>=0)
		{
			WEEKS_ONLY2(
			if (act.weekFlag & EVEN_WEEKS)
				UpdateContinuous(t, t+act.length+res.breakLen-1, res, 0);
			if (act.weekFlag & ODD_WEEKS)
				UpdateContinuous(t, t+act.length+res.breakLen-1, res, 1);
			,
			UpdateContinuous(t, t+act.length-1, res);
			)
		}
		)
	}
	ScoreFromSoftPreferences(t, t+act.length-1, act.timePreferences WEEKS_ARG(1));
}

void MySpace::ChooseResource(int pos)
{
	int act = info.top().act;
	int selno = size(info.top().res)-1;
	int res = info.top().res.top()[pos];
	journal_selection.push(make_pair(act, res));
	show(20,("Activity %d [%s] selects resource %d [%s]\n", ACT(act), RES(res)));
	grsel[ps.activities[act].grsel+selno] = res;
	RemoveResourceByAct(act, res);
	ScoreFromSoftPreferences(time[act].min, time[act].min+ps.activities[act].length-1, ps.resources[res].timePreferences
		WEEKS_ARG(ps.activities[act].weekFlag==ALL_WEEKS ? 1 : 2));
}

inline void MySpace::ScoreUpdated()
{
	if (bestScore<=myScore)
	{
		show(21,("Score to low -> fail\n"));
		throw failno=-1;
	}
}

void MySpace::UnuseActivity()
{
	int act = info.top().act;
	show(19,("Activity %d [%s] unused\n", ACT(act)));
	myScore += UNASSIGNED_SCORE;
	ScoreUpdated();
}

void MySpace::Select(int pos)
{
	show(20,("------------------------------------------\n"));
	if (state!=normal) return;
	try
	{
		if (pos==0)
		{
			ChooseTime(info.top().time);
			CREF(actS, ps.activities[info.top().act]);
			REPV(a, actS.requiredGroups)
			{
				if (state==failed) return;
				info.top().res.push(vector<int>());
				FindBestResource();
				ChooseResource(0);
			}
		}
		else
			UnuseActivity();
		info.push(Info());
	}
	catch(...)
	{
		show(20,("(failed %d)\n", failno));
		Fail();
	}
}

int MySpace::Select_All(int pos)
{
	if (state!=normal) return 0;
	try
	{
		if (AreWeSelectingTime())
		{
			int x = prototypeSolution.time[info.top().act];
			if (pos<0) pos = x<0 ? ps.noOfSlots : x;
			x = x>=0 ? x : searchSugested ? 0 : ps.noOfSlots;
			for (;;)
			{
				int p0 = pos;
				if (searchSugested)
				{
					if (pos==x) pos = ps.noOfSlots;
					else if (pos==ps.noOfSlots) pos = (x+1)%ps.noOfSlots;
					else pos = (pos+1)%ps.noOfSlots;
				}
				else
					pos = (pos+1)%(ps.noOfSlots+1);
				if (p0>=ps.noOfSlots)
				{
					UnuseActivity();
					break;
				}
				if (time[info.top().act].possible[p0]) 
				{
					ChooseTime(p0);
					break;
				}
			}
		}
		else
		{
			show(19,("Choosing resource\n"));
			if (pos<0) pos = 0; 
			ChooseResource(pos);
			++pos;
		}
		if (!used[info.top().act] || size(info.top().res)>=size(ps.activities[info.top().act].requiredGroups))
			info.push(Info());
		else
			info.top().res.push(vector<int>());
		return pos;
	}
	catch(...)
	{
		Fail();
		return 0;
	}
}

void MySpace::AssignActivities(int act, const vector<pair<int, int> > &actList)
{
	set<int> mySet, mySet2;
	FOREACH(ai, actList)
		mySet.insert(ai.first);
	mySet2.insert(act);
	partition(waiting, mySet, 0, size(waiting));
	partition(waiting, mySet2, 0, size(actList));
}

void MySpace::GenerateSolution(Solution &solution)
{
	MULTI_ONLY( lockSolution(0); )
	if (myScore>=*solution.bestScorePtr) goto finish;
	*solution.bestScorePtr = myScore;
	REP(a, ps.grselNo)
		solution.grsel[a] = FSgrsel[a];
	REPV(a, time)
	{
		solution.time[a] = time_sel[a];
		if (time_sel[a]<0)
			REPV(b, ps.activities[a].requiredGroups)
				solution.grsel[ps.activities[a].grsel+b] = -1;
	}
finish:
	MULTI_ONLY( unlockSolution(0); )
	return;
}

void MySpace::IHaveSolution()
{
	if (myScore>=bestScore || state!=succeded) return;
	show(10,("Found solution, score=%d\n", myScore));
	REPV(a, ps.activities)
		time_sel[a] = used[a] ? time[a].min : -1;
	FSgrsel = grsel;
	if (showScore)
	{
		fprintf(stderr2, "%d\n", myScore);
		fflush(stderr2);
	}
	GenerateSolution(bestSolution);
	FastShufle();
	GenerateSolution(bestSugestion);
}



void MySpace::BackToState(JournalState js DEBUG_ARG(const MySpace *sp))
{
	show(20,("Going back... "));
//	printf("Odtw %d\n", js.vpos);
	while (size(journal_selection)>js.selectionsize)
	{
		CREF(s, journal_selection.top());
		int a = s.first;
		if (s.second<0)
		{
			used[a] = false;
			CREF(act, ps.activities[a]);
			FOREACH(r, act.requiredResources)
				UndoRemoveResourceByAct(a, r);
		}
		else
			UndoRemoveResourceByAct(a, s.second);
		journal_selection.pop();
	}
	while (size(journal_values)>js.vpos)
	{
		CREF(p, journal_values.top());
		REF(t, time[p.first]);
		t.possible[p.second] = true;
		t.actualAt = moment;
		updateMin(t.min, p.second);
		updateMax(t.max, p.second);
		journal_values.pop();
	}
	while (size(journal_propagators)>js.ppos)
	{
		CREF(p, journal_propagators.top());
		time[p.first].propagators.push_back(p.second);
		journal_propagators.pop();
	}
	CONT_ONLY(
	while (size(journal_cont)>js.cpos)
	{
		CREF(c, journal_cont.top());
		cont_start WEEKS_ONLY([c.week]) [c.contres][c.day] = c.start;
		cont_stop WEEKS_ONLY([c.week]) [c.contres][c.day] = c.stop;
		journal_cont.pop();
	}
	)
	while (size(info)>js.infosize)
		info.pop();
	while (size(info.top().res)>js.inforessize)
		info.top().res.pop();
	myScore = js.myScore;
	randseed = js.randseed;
	noOfWaiting = js.noOfWaiting;
	while (!changedVar.empty()) changedVar.pop();
	++moment;
	state = normal;
	show(20,("%d done\n", myScore));
	
	#if ENABLE_DEBUG
	if (sp==NULL) return;
	assert(sp->propagators==propagators);
	assert(sp->changedVar.empty());
	assert(changedVar.empty());
	assert(sp->changedVar==changedVar);
	assert(size(sp->time)==size(time));
	REPV(ti, time)
	{
		CREF(t1,sp->time[ti]);
		CREF(t2,time[ti]);
		assert(t1.possible==t2.possible);
		set<Propagator*> s1, s2;
		FOREACH(a, t1.propagators) s1.insert(a);
		FOREACH(a, t2.propagators) s2.insert(a);
		assert(s1==s2);
		assert(t1.min==t2.min);
		assert(t1.max==t2.max);
	}
	assert(sp->state==state);
	assert(sp->journal_values==journal_values); 
	assert(sp->journal_propagators==journal_propagators);

	REPV(a,used) if (used[a] && a!=info.top().act)
	{
		CREF(act, ps.activities[a]);
		REPV(gi,act.requiredGroups)
		{
			if (sp->grsel[gi+act.grsel]!=grsel[gi+act.grsel])
				printf("%d %d %d %d\n", a, gi+act.grsel, sp->grsel[gi+act.grsel], grsel[gi+act.grsel]);
			assert(sp->grsel[gi+act.grsel]==grsel[gi+act.grsel]);
		}
	}
	set<int> w1, w2;
	FOREACH(a, sp->waiting) w1.insert(a);
	FOREACH(a, waiting) w2.insert(a);
	assert(w1==w2);
	assert(sp->noOfWaiting==noOfWaiting);
/*	REPV(a, availableRes)
		REPV(t, availableRes[a])
			if (!(sp->availableRes[a][t]==availableRes[a][t]))
				printf("%d %d %d %d %d %d %d %d\n", a, t, 
				availableRes[a][t].tab[0], availableRes[a][t].tab[1], availableRes[a][t].tab[2], 
				sp->availableRes[a][t].tab[0], sp->availableRes[a][t].tab[1], sp->availableRes[a][t].tab[2]);
*/	assert(sp->availableRes==availableRes);
	assert(sp->isGrresUsed==isGrresUsed);
	assert(sp->used==used);
	#if ENABLE_CONT
	assert(sp->cont_start==cont_start);
	assert(sp->cont_stop==cont_stop);
	#endif
	stack<Info> i1 = sp->info;
	VAR(i2, info);
	if (!i1.top().res.empty()||!i2.top().res.empty())
		assert(i1==i2);
	i1.pop(); i2.pop();
	assert(i1==i2);
/*	printf(":%d\n", size(i1));
	while (!i1.empty())
	{
		if (!(i1.top()==i2.top()))
			printf("%d\n", size(i1));
		i1.pop();
		i2.pop();
	}
	//assert(sp->info==info);		*/
	assert(sp->journal_selection==journal_selection);
	assert(sp->myScore==myScore);
//	assert(sp->time_sel==time_sel);
	assert(sp->randseed==randseed);
	#endif
}

#if ENABLE_DEBUG
void MySpace::CheckCurrentSolution()
{
	VAR(ps2, const_cast<ProblemSpecification &>(ps));
	REF(sol, ps2.solution);
	sol.resize(size(time_sel));
	REPV(a, time_sel)
	{
		sol[a].actNo = a;
		sol[a].startTime = time_sel[a];
		sol[a].usedResources.clear();
		if (time_sel[a]>=0)
		{
			CREF(act, ps.activities[a]);
			FOREACH(r, act.requiredResources)
				sol[a].usedResources.push_back(r);
			REPV(gi, act.requiredGroups)
				sol[a].usedResources.push_back(FSgrsel[act.grsel+gi]);
		}
	}
	int sc = ps2.CheckSolution();
	show(20,("%d\n", sc));
	assert(myScore == sc);
}
#endif

} // namespace Spaces

