/*
 *  Author: Pawel Parys <parys@mimuw.edu.pl> (2003-2004)
 *
 *  For licence see doc/LICENCE.
 */

#ifndef __PROPAGATORS_H
#define __PROPAGATORS_H

#include "space.h"

/* This module implements four basic propagators. All of them says something
 * about relation between activities number 'a1' and 'a2'. 'Disjoint' says
 * that the activities can not overlap. 'l1' and 'l2' are lenghts of the
 * activities (if break is needed between them, the lengths are increased by
 * length of the break). 'Before' says that the first activity ('l1' is its
 * length) should end before the second. 'SameDay' says that they should
 * start at the same day (start time / 'slotsPerDay' should be the same). 
 * 'Concurrently' says that the second should start not earlier than the
 * first and no more that 'maxDiff' after the first starts. */

namespace Propagators
{

class DisjointPropagator : public Propagator
{
	int a1, a2;
	int l1, l2;
public:
	DisjointPropagator(Spaces::Space &sp, int a10, int l10, int a20, int l20);
	virtual bool Propagate(Spaces::Space &sp, int actNo);
};

class BeforePropagator : public Propagator
{
	int a1, a2;
	int l1;
public:
	BeforePropagator(Spaces::Space &sp, int a10, int l10, int a20);
	virtual bool Propagate(Spaces::Space &sp, int actNo);
};

class SameDayPropagator : public Propagator
{
	int a1, a2;
	int slotsPerDay;
public:
	SameDayPropagator(Spaces::Space &sp, int a10, int a20, int slPerDay);
	virtual bool Propagate(Spaces::Space &sp, int actNo);
};

class ConcurrentlyPropagator : public Propagator
{
	int a1, a2;
	int maxDiff;
public:
	ConcurrentlyPropagator(Spaces::Space &sp, int a10, int a20, int maxD);
	virtual bool Propagate(Spaces::Space &sp, int actNo);
};

} // namespace Propagators

#endif
