/*
 *  Author: Pawel Parys <parys@mimuw.edu.pl> (2003-2004)
 *
 *  For licence see doc/LICENCE.
 */

#include "search.h"
#include "myspace.h"
#include "solution.h"
#include "multiproc.h"
#include "xxx.h"
#include <climits>

using namespace Spaces;

void doSearch(MySpace &space)
{
	for (;;)
	{
		space.PrepareSelection();
		switch(space.GetState())
		{
		case Space::failed: 
			yyerror_arg("Can not find solution (contradictory condicions)\n"
			"Failed on activity: %d[%s]", ACT(space.failno));
		case Space::succeded: 
			space.IHaveSolution();
			return;
		case Space::normal:
			{ 
				VAR(js, space.GetJournalState());
				space.Select(0);
				if (space.GetState()==Space::failed)
				{
					space.BackToState(js);
					space.Select(1);
				}
			}
		}
	}
}

int startTime, programStartTime;
int stopAfter = 5, stepsUnchanged = 25, timeLimit = -1, maxLevel, endReached = 1;

void doSearch_All_Rec(MySpace &space, int level)
{
	if (space.AreWeSelectingTime())
		show(19,("Level = %d, score = %d\n", level, space.myScore));
	if ((endReached && time(NULL)>=startTime+stopAfter) || time(NULL)>=startTime+2*stopAfter)
		throw -1;
	if (!level) endReached = 1;
	int alt = space.PrepareSelection_All();
	int newLev = level-space.AreWeSelectingTime();
	switch(space.GetState())
	{
	case Space::failed:
		show(20,("Fail\n"));
		break;
	case Space::succeded:
		space.IHaveSolution();
		break;
	case Space::normal:
		if (space.myScore>=bestScore)
			break;
		if (MySpace::searchSugested)
			alt = min(alt, 1+space.AreWeSelectingTime());
		for (int a = 0, pos = -1; a<alt; a++)
		{
			if (a==1 && level>maxLevel) maxLevel = level;
			VAR(js, space.GetJournalState());
			pos = space.Select_All(pos);
			doSearch_All_Rec(space, newLev);
			space.BackToState(js);
		}
		return;
	}
}

void doSearch_All(MySpace &space)
{
	try
	{
		startTime = time(NULL);
		maxLevel = 0;
		endReached = 0;
		doSearch_All_Rec(space, space.noOfWaiting);
	}
	catch(...)
	{
		show(10,("Time passed -> break (max level = %d)\n", maxLevel));
	}
}

int timeLeft()
{
	if (timeLimit>=0 && time(NULL)>=programStartTime+timeLimit)
	{
		show(10,("Time limit exceeded, ending...\n"));
		return 1;
	}
	else
		return 0;
}

void tryUseUnused(const ProblemSpecification &ps, MySpace &space)
{
	VAR(js, space.GetJournalState());
	int prevScore = INT_MAX;
	while (bestScore>0)
	{
		int noOfAct = size(ps.activities);
		show(10,("Starting new pass\n"));
		vector<int> unused;
		if (bestScore>=prevScore)
		{
			show(10,("score=%d, prev=%d\n", bestScore, prevScore));
			return;
		}
		prevScore = bestScore;
		show(10,("Current score is %d\n", prevScore));
		for (int a = 0; a<noOfAct; a++)
			if (bestSolution.time[a]<0)
				unused.push_back(a);
		for (int ai = 0; ai<size(unused); ai++)
		{
			MULTI_ONLY(
			if (noOfProcesses)
				tryChangeMultiproc(ps, unused[ai]);
			else
			)
				tryChange(ps, space, unused[ai]);
			space.BackToState(js);
			if (fastStop || timeLeft()) return;
		}
	}
}

void tryChange(const ProblemSpecification &ps, MySpace &space, int a)
{
	MULTI_ONLY( lockSolution(); )
	copySolution(prototypeSolution, bestSugestion, ps);
	MULTI_ONLY( unlockSolution(); )
	space.AssignActivities(a, ps.activities[a].conflictsWith);
	show(10,("Trying with activity %d [%s] (and %d friends)\n", ACT(a), size(ps.activities[a].conflictsWith)-1));
	MySpace::searchSugested = (ps.dependences.empty() && bestScore>*bestSugestion.bestScorePtr);
	doSearch_All(space);
}

int Random()
{
	static int seed = 1234;
	srand(seed);
	return seed = rand();
}

void performSearch(const ProblemSpecification &ps)
{
	try
	{
	show(10,("Creating space\n"));
	programStartTime = time(NULL);
	MySpace space;
	initializeMultiproc(ps, space);
	VAR(js, space.GetJournalState());
	if (hasInitialSolution)
		ps.SolutionToBestSol();
	else
	{
		show(10,("Searching solution\nFirst approach\n"));
		doSearch(space);
		space.BackToState(js);
	}
	show(10,("Trying to use unused activities\n"));
	tryUseUnused(ps, space);
	show(10,("Trying to change random activities\n"));
	
	for (int a = 0; a<stepsUnchanged && !fastStop && !timeLeft(); a++)
	{
		int prev = bestScore;
		if (bestScore>0)
		{
			int a = Random()%size(ps.activities);
			MULTI_ONLY(
			if (noOfProcesses)
				tryChangeMultiproc(ps, a);
			else
			)
				tryChange(ps, space, a);
			space.BackToState(js);
		}
		if (bestScore<prev)
			a = 0;
	}
	closeMultiproc();
	show(10,("Best score: %d\n", bestScore));
	}
	catch(int a)
	{
		yyerror_arg("Unexpected exception %d (internal error)", a);
	}
}
