/*
 *  Author: Pawel Parys <parys@mimuw.edu.pl> (2003-2004)
 *
 *  For licence see doc/LICENCE.
 */

#include "solution.h"
#include "xmlgenerator.h"
#include "problemspec.h"
#include "xml.h"
#include "tools.h"
#include "xxx.h"
#include <map>

/* Implements also problemspec.h ('ProblemSpecification::SolutionToBestSol'
 * method)" */

using namespace Analyser;
using namespace XML;
using namespace std;

Solution bestSolution, bestSugestion, prototypeSolution;

void copySolution(Solution &dest, const Solution &src, const ProblemSpecification &ps)
/* Use this function to copy 'Solution's. */
{
	memcpy(dest.time, src.time, sizeof(int)*(size(ps.activities)+ps.grselNo+1));
}

void addResource(const ProblemSpecification &ps, Elements *sons, int r)
/* It writes "<Resource>" element for resource number 'r' into 'sons'. */
{
	show(20,("requires resource %d [%s]\n", RES(r)));
	int s = size(sons->children);
	sons->children.resize(s+1);
	Element &resource = sons->children[s] = Element("Resource");
	resource.content = new TextContent(ps.resources[r].id.c_str());
}

void saveSolution(const ProblemSpecification &ps, FILE *file, Elements &main)
/* Save solution from 'bestSolution' global variable to 'file'. Call with 
 * empty XML tree 'main'. It will be used to store the generated tree. Then 
 * you will get it back. (It is then again analysed and checked by 
 * checksol.cc). */
{
	char buf[1024];
	Elements *sons, *sons0;
	main.children.resize(1);
	Element &timetable = main.children[0] = Element("Timetable");
	timetable.content = sons = new Elements();
	sons->children.resize(1);
	Element &solution = sons->children[0] = Element("Solution");
	solution.content = sons0 = new Elements();
	sons0->children.resize(size(ps.activities));
	REPV(a, ps.activities)
	{
		Element &activity = sons0->children[a] = Element("Activity");
		activity.attributes.push_back(Attribute("id", ps.activities[a].id));
		show(20,("Activity %d [%s], time %d\n", ACT(a), bestSolution.time[a]));
		if (bestSolution.time[a]>=0)
		{
			activity.content = sons = new Elements();
			sons->children.resize(2);
			Element &startTime = sons->children[0] = Element("StartTime");
			sprintf(buf, "%d", bestSolution.time[a]-bestSolution.time[a]/ps.slotsPerDay*ps.additionalTimeslots);
			startTime.content = new TextContent(buf);
			Element &usedResources = sons->children[1] = Element("UsedResources");
			usedResources.content = sons = new Elements();
			REPV(ri, ps.activities[a].requiredGroups)
			{
				show(20,("Activity %d [%s] (grsel %d) ", ACT(a), ri));
				addResource(ps, sons, bestSolution.grsel[ps.activities[a].grsel+ri]);
			}
			REPV(ri, ps.activities[a].requiredResources)
			{
				show(20,("Activity %d [%s] (reqRes %d) ", ACT(a), ri));
				addResource(ps, sons, ps.activities[a].requiredResources[ri]);
			}
		}
	}
	GenerateXML(file, main);
}

void ProblemSpecification::SolutionToBestSol() const
/* This function copies 'solution' stored in ProblemSpecification to
 * 'bestSolution' global variable. (It needs also some coversion.)
 * 
 * IMPORTANT (MULTIPROC): It do not lock 'bestSolution' while writing to it.
 * Use only while other process are not working (they are already created,
 * but they have not got any orders yet). */
{
	REPV(sai, solution)
	{
		const SolActivity &sa = solution[sai];
		const Activity &act = activities[sa.actNo];
		bestSolution.time[sa.actNo] = sa.startTime;
		if (sa.startTime<0)
		{
			REPV(gi, act.requiredGroups)
				bestSolution.grsel[act.grsel+gi] = -1;
			continue;
		}
		map<int,int> m;
		REPV(gi, act.requiredGroups)
		{
			int g = act.requiredGroups[gi];
			REPV(ri, groups[g].resources)
				m.insert(pair<int,int>(groups[g].resources[ri], gi));
		}
		REPV(ri, sa.usedResources)
		{
			int r = sa.usedResources[ri];
			map<int,int>::iterator found = m.find(r);
			if (found==m.end())
				continue;
			int gi = found->second;
			bestSolution.grsel[act.grsel+gi] = r;
		}
	}
	bestScore = initialSolutionScore;
	copySolution(bestSolution, bestSugestion, *this);
}
