/*
 *  Author: Pawel Parys <parys@mimuw.edu.pl> (2003-2004)
 *
 *  For licence see doc/LICENCE.
 */

#ifndef __SOLUTION_H
#define __SOLUTION_H

#include <cstdio>

/* Variables 'hasInitialSolution' and 'initialSolutionScore' are declared in
 * 'main.cc' */

struct Solution
{
	int *time;
	int *grsel;
	int *bestScorePtr;
};
/* This is the structure with solution. When activity is unassigned time is
 * negative. 
 * 
 * IMPORTANT: We assume that 'time', 'grsel', 'bestScorePtr' point to
 * consecutive memory areas. */

extern Solution bestSolution, bestSugestion, prototypeSolution;
/* 'bestSolution' is best reached solution which is correct, comes from basic
 * algorithm. 'bestSugestion' may be even better than 'bestSolution', but it
 * do not have to be correct (when there are 'Dependences'). It comes from
 * fastShufle algorithm. 'prototypeSolution' is used as a templatate where
 * looking for better solution (it is the same that 'bestSuggestion'). 
 * 
 * The 'best...' variables are set after the first solution is found (or
 * after the solution is loaded from file). 'prototypeSolution' is set while
 * starting a search. 
 * 
 * MULTIPROC: 'best...' variables are stored in shared memory (common for
 * all processes). 'prototypeSolution' is seperate for each process.
 * 
 * IMPORTANT (MULTIPROC): before any operation on 'best...' you have to call
 * 'lockSolution' and after the operation 'unlockSolution'. When modifying
 * use 0 argument (default 1 means only reading). */

extern bool hasInitialSolution;	// set when there is solution loaded from file
extern int initialSolutionScore;	// score of that solution

#define bestScore (*bestSolution.bestScorePtr)
#define bestSugScore (*bestSugestion.bestScorePtr)
// macros for accessing score of solutions in bestSolution / bestSugestion

namespace Analyser { class ProblemSpecification; }
namespace XML { class Elements; }

void saveSolution(const Analyser::ProblemSpecification &ps, FILE *file, XML::Elements &outDesc);
/* Save solution from 'bestSolution' global variable to 'file'. Call with 
 * empty XML tree 'main'. It will be used to store the generated tree. Then 
 * you will get it back. (It is then again analysed and checked by 
 * checksol.cc). */

void copySolution(Solution &dest, const Solution &src, const Analyser::ProblemSpecification &ps);
/* Use this function to copy 'Solution's. */

#endif
