/*
 *  Author: Pawel Parys <parys@mimuw.edu.pl> (2003-2004)
 *
 *  For licence see doc/LICENCE.
 */

#ifndef __TOOLS_H
#define __TOOLS_H

#include <vector>
#include <set>

/* Two basic functions operating on vectors. */

using namespace std;

template<class T> inline void removeV(vector<T> &v, int pos)
/* Removes element on position 'pos' from vector, copies last element to this
 * position. */
{
	v[pos]=v[size(v)-1];
	v.resize(size(v)-1);
}

template<class T> void partition(vector<T> &vec, const set<T> &first, int from, int size)
/* It works on subvector of 'vec' beginning at 'from' of size 'size'. Elements
 * of this subvector which are in 'first' are moved to the beginning of the
 * subvector, the others are moved after them. */
{
	int l = from, r = from+size-1; 
	while (l<r)
	{
		if (first.find(vec[l])!=first.end()) { l++; continue; }
		if (first.find(vec[r])==first.end()) { r--; continue; }
		swap(vec[l], vec[r]); l++; r--;
	}
}

#endif
