/*
 *  Author: Pawel Parys <parys@mimuw.edu.pl> (2003-2004)
 *
 *  For licence see doc/LICENCE.
 */

#include <cstdarg>
#include <cstdio>
#include "xml.h"
#include "xxx.h"

namespace XML
{

FILE *file;	// we are writing to this file 

int first;	// are we on the very top level?
		//(on this level we do not need to indent)

int indent;	// current number of spaces at the beginning of every line
int enter;	// is current line empty (except first - for first line = 0)

/* The following four procedures will be used together to writing XML to file.
 *
 * 'myprintf' is like printf, but when it is on beginning of a line it prints
 * some number of spaces before the string (set by Indent() and Unindent()).
 * We consider that format do not contain '\n'. 
 * 
 * 'Enter' goes to next line (only if line is not empty, when line is empty do
 * nothing).
 * 
 * 'Indent' / 'Unindent' increases / decreases the number of spaces on
 * beginning of each line (by 2). When used after 'Enter' and before first 
 * 'myprintf' it affects also the current line. */

void myprintf(const char *format, ...)
{
	if (enter)
	{
		REP(a, indent)
			fprintf(file, " ");
		enter = 0;
	}
        va_list ap;
        va_start(ap, format);
        vfprintf(file, format, ap);
        va_end(ap);
}

void Enter()
{
	if (!enter) 
		fprintf(file, "\n");
	enter = 1;
}

inline void Indent()
{
	indent+=2;
}

inline void Unindent()
{
	indent-=2;
}

/* 'Show' methods do show appropiate objects. */

void Elements::Show() const
{
	REPV(a, children)
		children[a].Show();
}

void Attribute::Show() const
{
	myprintf(" %s=\"%s\"", name.c_str(), value.c_str());
}

void Element::Show() const
{
	int flocal = first;
	if (!first) Indent(); else first = 0;
	Enter();
	myprintf("<%s", name.c_str());
	REPV(a, attributes)
		attributes[a].Show();
	if (content==NULL)
		myprintf("/>");
	else
	{
		myprintf(">");
		content->Show();
		myprintf("</%s>", name.c_str());
	}
	Enter();
	if (!flocal) Unindent(); else first = 1;
}

void TextContent::Show() const
{
	myprintf("%s", value.c_str());
}

void GenerateXML(FILE *f, const Elements &output)
/* Write XML structure 'output' to file 'f'. */
{
	file = f;
	first = 1;
	indent = 0;
	enter = 0;
	fprintf(file, "<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
	Enter();
	output.Show();
}

} // namespace XML
